# Copyright DST Group. Licensed under the MIT license.
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.remote_code_execution import RemoteCodeExecution


class RubyOnRails(RemoteCodeExecution):
    def __init__(self):
        super().__init__()

    def sim_execute(self, state):
        pass
