# Copyright DST Group. Licensed under the MIT license.
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.internal_enumeration import InternalEnumeration


class NetworkInfoEnumeration(InternalEnumeration):
    def __init__(self):
        super().__init__()

    def sim_execute(self):
        pass
