from csle_cyborg.shared.actions.action import Sleep, InvalidAction
from csle_cyborg.shared.actions.abstract_actions.remove import Remove
from csle_cyborg.shared.actions.abstract_actions.analyse import Analyse
from csle_cyborg.shared.actions.abstract_actions.discover_network_services import DiscoverNetworkServices
from csle_cyborg.shared.actions.abstract_actions.discover_remote_systems import DiscoverRemoteSystems
from csle_cyborg.shared.actions.abstract_actions.exploit_remote_service import ExploitRemoteService
from csle_cyborg.shared.actions.abstract_actions.impact import Impact
from csle_cyborg.shared.actions.abstract_actions.misinform import Misinform
from csle_cyborg.shared.actions.abstract_actions.monitor import Monitor
from csle_cyborg.shared.actions.abstract_actions.privilege_escalate import PrivilegeEscalate
from csle_cyborg.shared.actions.abstract_actions.remove import Remove
from csle_cyborg.shared.actions.abstract_actions.restore import Restore
from csle_cyborg.shared.actions.agent_actions.agent_action import AgentAction
from csle_cyborg.shared.actions.agent_actions.agent_sleep import AgentSleep
from csle_cyborg.shared.actions.agent_actions.create_session import CreateSession
from csle_cyborg.shared.actions.agent_actions.get_initial_agent_observation import GetInitialAgentObservation
from csle_cyborg.shared.actions.agent_actions.list_sessions import ListSessions
from csle_cyborg.shared.actions.concrete_actions.blue_keep import BlueKeep
from csle_cyborg.shared.actions.concrete_actions.concrete_action import ConcreteAction
from csle_cyborg.shared.actions.concrete_actions.decoy_apache import DecoyApache
from csle_cyborg.shared.actions.concrete_actions.decoy_femitter import DecoyFemitter
from csle_cyborg.shared.actions.concrete_actions.decoy_haraka_smpt import DecoyHarakaSMPT
from csle_cyborg.shared.actions.concrete_actions.decoy_smss import DecoySmss
from csle_cyborg.shared.actions.concrete_actions.decoy_sshd import DecoySSHD
from csle_cyborg.shared.actions.concrete_actions.decoy_svchost import DecoySvchost
from csle_cyborg.shared.actions.concrete_actions.decoy_tomcat import DecoyTomcat
from csle_cyborg.shared.actions.concrete_actions.decoy_vsftpd import DecoyVsftpd
from csle_cyborg.shared.actions.concrete_actions.density_scout import DensityScout
from csle_cyborg.shared.actions.concrete_actions.escalate_action import EscalateAction
from csle_cyborg.shared.actions.concrete_actions.eternal_blue import EternalBlue
from csle_cyborg.shared.actions.concrete_actions.exploit_action import ExploitAction
from csle_cyborg.shared.actions.concrete_actions.ftp_directory_traversal import FTPDirectoryTraversal
from csle_cyborg.shared.actions.concrete_actions.haraka_rce import HarakaRCE
from csle_cyborg.shared.actions.concrete_actions.http_rfi import HTTPRFI
from csle_cyborg.shared.actions.concrete_actions.http_srfi import HTTPSRFI
from csle_cyborg.shared.actions.concrete_actions.juicy_potato import JuicyPotato
from csle_cyborg.shared.actions.concrete_actions.pingsweep import Pingsweep
from csle_cyborg.shared.actions.concrete_actions.port_scan import Portscan
from csle_cyborg.shared.actions.concrete_actions.remote_code_execution_on_smtp import RemoteCodeExecutionOnSMTP
from csle_cyborg.shared.actions.concrete_actions.restore_from_backup import RestoreFromBackup
from csle_cyborg.shared.actions.concrete_actions.sig_check import SigCheck
from csle_cyborg.shared.actions.concrete_actions.sql_injection import SQLInjection
from csle_cyborg.shared.actions.concrete_actions.ssh_brute_force import SSHBruteForce
from csle_cyborg.shared.actions.concrete_actions.stop_process import StopProcess
from csle_cyborg.shared.actions.concrete_actions.stop_service import StopService
from csle_cyborg.shared.actions.concrete_actions.v4l2_kernel_exploit import V4L2KernelExploit
from csle_cyborg.shared.actions.game_actions.create_agent import CreateAgent
from csle_cyborg.shared.actions.game_actions.game_action import GameAction
from csle_cyborg.shared.actions.game_actions.game_echo import GameEcho
from csle_cyborg.shared.actions.game_actions.game_sleep import GameSleep
from csle_cyborg.shared.actions.game_actions.get_true_state import GetTrueState
from csle_cyborg.shared.actions.game_actions.list_agents import ListAgents
from csle_cyborg.shared.actions.game_actions.list_all_sessions import ListAllSessions
from csle_cyborg.shared.actions.game_actions.reset_game import ResetGame
from csle_cyborg.shared.actions.global_actions.create_game import CreateGame
from csle_cyborg.shared.actions.global_actions.global_action import GlobalAction
from csle_cyborg.shared.actions.global_actions.global_echo import GlobalEcho
from csle_cyborg.shared.actions.global_actions.global_sleep import GlobalSleep
from csle_cyborg.shared.actions.global_actions.list_games import ListGames
from csle_cyborg.shared.actions.global_actions.shutdown_game import ShutdownGame
from csle_cyborg.shared.actions.green_actions.green_connection import GreenConnection
from csle_cyborg.shared.actions.green_actions.green_ping_sweep import GreenPingSweep
from csle_cyborg.shared.actions.green_actions.green_port_scan import GreenPortScan
from csle_cyborg.shared.actions.local_shell_actions.local_shell_sleep import LocalShellSleep
from csle_cyborg.shared.actions.local_shell_actions.local_shell_echo import LocalShellEcho
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.get_pid import GetPid
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.get_shell import GetShell
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.get_uid import GetUid
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.local_time import LocalTime
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.meterpreter_action import MeterpreterAction
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.meterpreter_ip_config import MeterpreterIPConfig
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.meterpreter_ps import MeterpreterPS
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.meterpreter_reboot import MeterpreterReboot
from csle_cyborg.shared.actions.msf_actions_folder.meterpreter_actions_folder.sys_info import SysInfo
from csle_cyborg.shared.actions.msf_actions_folder.msf_persistence_folder.service_persistence_windows import ServicePersistenceWindows
from csle_cyborg.shared.actions.msf_actions_folder.msf_privilege_escalation_folder.msf_privilege_escalation import MSFPrivilegeEscalation
from csle_cyborg.shared.actions.msf_actions_folder.msf_scanner_folder.msf_scanner import MSFScanner
from csle_cyborg.shared.actions.msf_actions_folder.msf_scanner_folder.msf_ping_sweep import MSFPingsweep
from csle_cyborg.shared.actions.msf_actions_folder.msf_scanner_folder.msf_port_scan import MSFPortscan
from csle_cyborg.shared.actions.msf_actions_folder.msf_scanner_folder.tomcat_credential_scanner import TomcatCredentialScanner
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.ms17_010_psexec import MS17_010_PSExec
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.msf_eternal_blue import MSFEternalBlue
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.ps_exec import PSExec
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.remote_code_execution import RemoteCodeExecution
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.ruby_on_rails import RubyOnRails
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.samba_usermap_script import SambaUsermapScript
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.ssh_login_exploit import SSHLoginExploit
from csle_cyborg.shared.actions.msf_actions_folder.remote_code_execution_folder.tomcat_exploit import TomcatExploit
from csle_cyborg.shared.actions.msf_actions_folder.msf_action import MSFAction
from csle_cyborg.shared.actions.msf_actions_folder.msf_auto_route import MSFAutoroute
from csle_cyborg.shared.actions.msf_actions_folder.upgrade_to_meterpreter import UpgradeToMeterpreter
from csle_cyborg.shared.actions.shell_actions_folder.delete_file_windows import DeleteFileWindows
from csle_cyborg.shared.actions.shell_actions_folder.delete_file_linux import DeleteFileLinux
from csle_cyborg.shared.actions.shell_actions_folder.find_flag import FindFlag
from csle_cyborg.shared.actions.shell_actions_folder.kill_process_linux import KillProcessLinux
from csle_cyborg.shared.actions.shell_actions_folder.kill_process_windows import KillProcessWindows
from csle_cyborg.shared.actions.shell_actions_folder.shell_action import ShellAction
from csle_cyborg.shared.actions.shell_actions_folder.shell_echo import ShellEcho
from csle_cyborg.shared.actions.shell_actions_folder.shell_ps import ShellPS
from csle_cyborg.shared.actions.shell_actions_folder.shell_sleep import ShellSleep
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.add_user_linux import AddUserLinux
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.account_manipulation import AccountManipulation
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.add_user_windows import AddUserWindows
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.disable_user_windows import DisableUserWindows
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.disable_user_linux import DisableUserLinux
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.remove_user_from_group_windows import RemoveUserFromGroupWindows
from csle_cyborg.shared.actions.shell_actions_folder.account_manipulation_folder.remove_user_from_group_linux import RemoveUserFromGroupLinux
from csle_cyborg.shared.actions.shell_actions_folder.credential_harvesting_folder.credential_harvesting import CredentialHarvesting
from csle_cyborg.shared.actions.shell_actions_folder.credential_harvesting_folder.read_passwd_file import ReadPasswdFile
from csle_cyborg.shared.actions.shell_actions_folder.credential_harvesting_folder.read_shadow_file import ReadShadowFile
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.internal_enumeration import InternalEnumeration
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.host_info_enumeration_folder.host_info_enumeration import HostInfoEnumeration
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.host_info_enumeration_folder.system_info import SystemInfo
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.host_info_enumeration_folder.uname import Uname
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.network_info_enumeration_folder.if_config import IFConfig
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.network_info_enumeration_folder.ip_config import IPConfig
from csle_cyborg.shared.actions.shell_actions_folder.internal_enumeration_folder.network_info_enumeration_folder.network_info_enumeration import NetworkInfoEnumeration
from csle_cyborg.shared.actions.shell_actions_folder.network_scan_folder.network_scan import NetworkScan
from csle_cyborg.shared.actions.shell_actions_folder.network_scan_folder.nmap_scan import NmapScan
from csle_cyborg.shared.actions.shell_actions_folder.network_scan_folder.ping_sweep import PingSweep
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.credential_access_folder.brute_force_access_folder.brute_force_access import BruteForceAccess
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.credential_access_folder.brute_force_access_folder.ssh_hydra_brute_force import SSHHydraBruteForce
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.credential_access_folder.credential_access import CredentialAccess
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.credential_access_folder.ssh_access import SSHAccess
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.netcat_connect import NetcatConnect
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.open_connection import OpenConnection
from csle_cyborg.shared.actions.shell_actions_folder.open_connection_folder.smb_anonymous_connection import SMBAnonymousConnection
from csle_cyborg.shared.actions.shell_actions_folder.persistence_folder.persistence import Persistence
from csle_cyborg.shared.actions.shell_actions_folder.persistence_folder.schtasks import Schtasks
from csle_cyborg.shared.actions.shell_actions_folder.service_manipulation_folder.service_manipulation import ServiceManipulation
from csle_cyborg.shared.actions.shell_actions_folder.service_manipulation_folder.shell_stop_service import ShellStopService
from csle_cyborg.shared.actions.shell_actions_folder.service_manipulation_folder.start_service import StartService
from csle_cyborg.shared.actions.shell_actions_folder.shell_privilege_escalation_folder.shell_privilege_escalation import ShellPrivilegeEscalation
from csle_cyborg.shared.actions.shell_actions_folder.shell_privilege_escalation_folder.dirty_cow_privilege_escalation import DirtyCowPrivilegeEscalation
from csle_cyborg.shared.actions.shell_actions_folder.shell_privilege_escalation_folder.linux_kernel_privilege_escalation import LinuxKernelPrivilegeEscalation
from csle_cyborg.shared.actions.shell_actions_folder.delete_file_linux import DeleteFileLinux
from csle_cyborg.shared.actions.shell_actions_folder.delete_file_windows import DeleteFileWindows
from csle_cyborg.shared.actions.shell_actions_folder.find_flag import FindFlag
from csle_cyborg.shared.actions.shell_actions_folder.kill_process_linux import KillProcessLinux
from csle_cyborg.shared.actions.shell_actions_folder.kill_process_windows import KillProcessWindows
from csle_cyborg.shared.actions.shell_actions_folder.shell_action import ShellAction
from csle_cyborg.shared.actions.shell_actions_folder.shell_echo import ShellEcho
from csle_cyborg.shared.actions.shell_actions_folder.shell_ps import ShellPS
from csle_cyborg.shared.actions.shell_actions_folder.shell_sleep import ShellSleep
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_alerts import GetAlerts
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_file_info import GetFileInfo
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_host_list import GetHostList
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_local_groups import GetLocalGroups
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_os_info import GetOSInfo
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_process_info import GetProcessInfo
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_process_list import GetProcessList
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_process_list_for_username import GetProcessListForUsername
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_user_info import GetUserInfo
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_user_ntds import GetUserNTDS
from csle_cyborg.shared.actions.velociraptor_actions_folder.get_users import GetUsers
from csle_cyborg.shared.actions.velociraptor_actions_folder.host_monitoring_action import HostMonitoringAction
from csle_cyborg.shared.actions.velociraptor_actions_folder.kill_process_name import KillProcessName
from csle_cyborg.shared.actions.velociraptor_actions_folder.kill_process_pid import KillProcessPID
from csle_cyborg.shared.actions.velociraptor_actions_folder.velociraptor_action import VelociraptorAction
from csle_cyborg.shared.actions.velociraptor_actions_folder.velociraptor_artifact_action import VelociraptorArtifactAction
from csle_cyborg.shared.actions.velociraptor_actions_folder.velociraptor_disable_user import VelociraptorDisableUser
from csle_cyborg.shared.actions.velociraptor_actions_folder.velociraptor_poll import VelociraptorPoll
from csle_cyborg.shared.actions.velociraptor_actions_folder.velociraptor_sleep import VelociraptorSleep
from csle_cyborg.shared.actions.action import Action
from csle_cyborg.shared.actions.session_action import SessionAction